@@ -171,6 +171,35 @@ test_that("subsetting,XcmsExperiment works", {
     expect_true(length(res) == (length(xmse) - 1))
     ref <- xmse[c(2, 3)]
     expect_equal(chromPeaks(res), chromPeaks(ref))
+
+    ## subsetting with alignment results.
+    tmp <- adjustRtime(xmseg, PeakGroupsParam(span = 0.4))
+    a <- tmp[3, keepAdjustedRtime = TRUE]
+    b <- tmp[3, keepAdjustedRtime = FALSE]
+    expect_true(length(a@processHistory) > length(b@processHistory))
+    expect_false(all(rtime(a) == rtime(b)))
+    expect_false(all(chromPeaks(a)[, "rt"] == chromPeaks(b)[, "rt"]))
+
+    ## subset in arbitrary order
+    a <- tmp[c(3, 1), keepAdjustedRtime = TRUE]
+    a_rt <- split(rtime(a, adjusted = TRUE), spectraSampleIndex(a))
+    tmp_rt <- split(rtime(tmp, adjusted = TRUE), spectraSampleIndex(tmp))
+    expect_equal(a_rt[[1L]], tmp_rt[[3L]])
+    expect_equal(a_rt[[2L]], tmp_rt[[1L]])
+    a_pks <- split.data.frame(chromPeaks(a), chromPeaks(a)[, "sample"])
+    tmp_pks <- split.data.frame(chromPeaks(tmp), chromPeaks(tmp)[, "sample"])
+    expect_equal(a_pks[[1L]][, 1:8], tmp_pks[[3L]][, 1:8])
+    expect_equal(a_pks[[2L]][, 1:8], tmp_pks[[1L]][, 1:8])
+
+    a <- tmp[c(3, 1), keepAdjustedRtime = FALSE]
+    a_rt <- split(rtime(a), spectraSampleIndex(a))
+    tmp_rt <- split(rtime(tmp, adjusted = FALSE), spectraSampleIndex(tmp))
+    expect_equal(a_rt[[1L]], tmp_rt[[3L]])
+    expect_equal(a_rt[[2L]], tmp_rt[[1L]])
+    a_pks <- split.data.frame(chromPeaks(a), chromPeaks(a)[, "sample"])
+    tmp_pks <- split.data.frame(chromPeaks(xmse), chromPeaks(xmse)[, "sample"])
+    expect_equal(a_pks[[1L]][, 1:8], tmp_pks[[3L]][, 1:8])
+    expect_equal(a_pks[[2L]][, 1:8], tmp_pks[[1L]][, 1:8])
 })
 
 test_that("filterRt,XcmsExperiment works", {
@@ -388,6 +417,40 @@ test_that(".xmse_group_cpeaks works", {
     expect_true(is.list(res$peakidx))
 })
 
+test_that("groupChromPeaks,XcmsExperiment,PeakDensityParam minFraction works", {
+    a <- as(loadXcmsData(), "MsExperiment")[c(1, 2, 7, 8)]
+    a <- findChromPeaks(
+        a, param = CentWaveParam(ppm = 5, peakwidth = c(5, 20),
+                                 prefilter = c(3, 1000), mzdiff = 0.001))
+    pdp <- PeakDensityParam(sampleGroups = rep(1, 4), bw = 8, binSize = 0.015,
+                            minFraction = 0.4, maxFeatures = 100)
+    b <- groupChromPeaks(a, pdp)
+    mzr <- c(278.9, 279.1)
+    idx <- which(MsCoreUtils::between(featureDefinitions(b)$mzmed, mzr))
+    tmp <- featureValues(b)[idx, ]
+    expect_true(nrow(tmp) == 2)
+    expect_true(sum(is.na(tmp[1, ])) <= 2)
+    expect_true(sum(is.na(tmp[1, ])) <= 2)
+
+    chr_b <- chromatogram(b, mz = mzr)
+    ## these two plots have to be the same
+    plotChromPeakDensity(chr_b, simulate = FALSE)
+    plotChromPeakDensity(chr_b, simulate = TRUE, param = pdp)
+
+    pdp <- PeakDensityParam(sampleGroups = rep(1, 4), bw = 8, binSize = 0.015,
+                            minFraction = 0.6, maxFeatures = 100)
+    b <- groupChromPeaks(a, pdp)
+    idx <- which(MsCoreUtils::between(featureDefinitions(b)$mzmed, mzr))
+    tmp <- featureValues(b)[idx, , drop = FALSE]
+    expect_true(nrow(tmp) == 1)
+    expect_true(sum(is.na(tmp[1, ])) <= 3)
+
+    chr_b <- chromatogram(b, mz = mzr)
+    ## these two plots have to be the same
+    plotChromPeakDensity(chr_b, simulate = FALSE)
+    plotChromPeakDensity(chr_b, simulate = TRUE, param = pdp)
+})
+
 test_that("groupChromPeaks,XcmsExperiment and related things work", {
     ## PeakDensityParam
     expect_false(hasFeatures(xmse))
@@ -1179,6 +1242,53 @@ test_that("featureSpectra works", {
     expect_true(length(res[[2L]]) < length(res_2[[2L]]))
     expect_true(all(res[[2L]]$chrom_peak_id %in% res_2[[2L]]$chrom_peak_id))
     expect_equal(unique(res[[2L]]$feature_id), unique(res_2[[2L]]$feature_id))
+
+    ## featureSpectra with and without `skipFilled`.
+    res <- featureSpectra(xmseg, msLevel = 1L)
+    res_2 <- featureSpectra(xmseg, msLevel = 1L, skipFilled = TRUE)
+    expect_equal(rtime(res), rtime(res_2))
+
+    a <- loadXcmsData("xmse")
+    fts <- rownames(featureDefinitions(a))[1:20]
+    fvals <- featureValues(a, filled = FALSE)[1:20, ]
+    ref <- featureSpectra(a, msLevel = 1L, skipFilled = FALSE,
+                          features = fts, method = "closest_rt")
+    res <- featureSpectra(a, msLevel = 1L, skipFilled = TRUE,
+                          features = fts, method = "closest_rt")
+    expect_true(length(res) < length(ref))
+    ## Feature 7: 3 detected, 5 gap-filled peaks.
+    ## Number of spectra per feature matches the non-NA feature values?
+    f7 <- res[res$feature_id == fts[7L]]
+    expect_equal(length(f7), sum(!is.na(fvals[7, ])))
+    pks_7 <- chromPeaks(a)[featureDefinitions(a)$peakidx[[7L]], ]
+    expect_equal(f7$chrom_peak_id, rownames(pks_7)[!is.na(pks_7[, "intb"])])
+    expect_equal(basename(dataOrigin(f7)), colnames(fvals)[!is.na(fvals[7, ])])
+
+    ## Are the spectra reported by skipFilled = TRUE the same as without?
+    f7_ref <- ref[ref$feature_id == fts[7L]]
+    expect_equal(length(f7_ref), ncol(fvals))
+    expect_true(all(paste0(f7$dataOrigin, f7$acquisitionNum) %in%
+                    paste0(f7_ref$dataOrigin, f7_ref$acquisitionNum)))
+
+    ## Feature 2: 8 detected peaks.
+    f2 <- res[res$feature_id == fts[2L]]
+    f2_ref <- ref[ref$feature_id == fts[2L]]
+    expect_equal(length(f2), length(f2_ref))
+    expect_equal(paste0(f2$dataOrigin, f2$acquisitionNum),
+                 paste0(f2_ref$dataOrigin, f2_ref$acquisitionNum))
+
+    ## Feature 3: 7 detected, one gap-filled
+    f3 <- res[res$feature_id == fts[3L]]
+    expect_equal(length(f3), sum(!is.na(fvals[3, ])))
+    pks_3 <- chromPeaks(a)[featureDefinitions(a)$peakidx[[3L]], ]
+    expect_equal(f3$chrom_peak_id, rownames(pks_3)[!is.na(pks_3[, "intb"])])
+    expect_equal(basename(dataOrigin(f3)), colnames(fvals)[!is.na(fvals[3, ])])
+
+    ## Are the spectra reported by skipFilled = TRUE the same as without?
+    f3_ref <- ref[ref$feature_id == fts[3L]]
+    expect_equal(length(f3_ref), ncol(fvals))
+    expect_true(all(paste0(f3$dataOrigin, f3$acquisitionNum) %in%
+                    paste0(f3_ref$dataOrigin, f3_ref$acquisitionNum)))
 })
 
 test_that("chromatogram,XcmsExperiment and .xmse_extract_chromatograms_old", {
@@ -1457,6 +1567,26 @@ test_that("findChromPeaksIsolationWindow, etc, MsExperiment works", {
     expect_true(nrow(chromPeaks(res)) < nrow(chromPeaks(a)))
     expect_true(all(chromPeakData(res)$isolationWindowLowerMz < 301))
     expect_true(all(chromPeakData(res)$isolationWindowUpperMz > 301))
+
+    ## With an XcmsExperimentHdf5
+    tf <- tempfile()
+    a_h5 <- .xcms_experiment_to_hdf5(a, tf)
+    expect_true(hasChromPeaks(a_h5, 1L))
+    expect_true(hasChromPeaks(a_h5, 2L))
+
+    res <- filterIsolationWindow(a_h5)
+    expect_equal(chromPeaks(res), chromPeaks(a_h5))
+
+    res <- filterIsolationWindow(a_h5, mz = 301)
+    expect_true(length(spectra(res)) < length(spectra(mse_dia)))
+    expect_true(all(isolationWindowLowerMz(res@spectra) < 301))
+    expect_true(all(isolationWindowUpperMz(res@spectra) > 301))
+
+    expect_true(nrow(chromPeaks(res)) < nrow(chromPeaks(a)))
+    expect_true(all(chromPeakData(res)$isolationWindowLowerMz < 301))
+    expect_true(all(chromPeakData(res)$isolationWindowUpperMz > 301))
+
+    rm(tf)
 })
 
 test_that("chromPeaksChromatograms,XcmsExperiment works", {
@@ -1531,3 +1661,54 @@ test_that("c,XcmsExperiment works", {
     expect_true(length(res) == length(a) * 2)
     expect_true(nrow(chromPeaks(res)) == nrow(chromPeaks(a)) * 2)
 })
+
+test_that("adjustRtime,XcmsExperiment,LamaParama works", {
+    pks <- chromPeaks(xmse)[1:100, c("mz", "rt")]
+    lp <- LamaParama(lamas = cbind(pks[, 1], pks[, 2] + 4),
+                     toleranceRt = 10, tolerance = 0.2)
+
+    expect_error(adjustRtime(as(mse, "XcmsExperiment"), lp),
+                 "detected chromPeaks")
+    lp <- matchLamasChromPeaks(xmse, lp)
+    lp@rtMap <- lp@rtMap[1:2]
+    expect_error(adjustRtime(xmse, lp), "Mismatch between the number")
+
+    lp <- matchLamasChromPeaks(xmse, lp)
+    lp@rtMap[[2L]] <- lp@rtMap[[2]][1:6, ]
+
+    expect_warning(res <- adjustRtime(xmse, lp, BPPARAM = SerialParam()),
+                   "for sample 2")
+    expect_s4_class(res, "XcmsExperiment")
+    ## No adjustment for 2nd sample
+    pks <- chromPeaks(xmse)
+    res_pks <- chromPeaks(res)
+    expect_equal(res_pks[res_pks[, "sample"] == 2, "rt"],
+                 pks[pks[, "sample"] == 2, "rt"])
+    expect_true(all(res_pks[res_pks[, "sample"] != 2, "rt"] !=
+                    pks[pks[, "sample"] != 2, "rt"]))
+    res_rt <- split(rtime(res), fromFile(res))
+    rt <- split(rtime(xmse), fromFile(xmse))
+    expect_equal(res_rt[[2]], rt[[2]])
+    expect_true(all(res_rt[[1]] != rt[[1]]))
+    expect_true(all(res_rt[[3]] != rt[[3]]))
+})
+
+test_that("chromPeakData,XcmsExperiment works with parameter columns", {
+    ref <- chromPeakData(xmse)
+    res <- chromPeakData(xmse, columns = "is_filled")
+    expect_s4_class(res, "DataFrame")
+    expect_true(ncol(res) == 1L)
+    expect_equal(colnames(res), "is_filled")
+    expect_equal(res$is_filled, ref$is_filled)
+
+    ref <- chromPeakData(xmse, msLevel = 2L)
+    expect_s4_class(ref, "DataFrame")
+    expect_equal(colnames(ref), c("ms_level", "is_filled"))
+    expect_true(nrow(ref) == 0L)
+    res <- chromPeakData(xmse, msLevel = 2L, columns = "is_filled")
+    expect_s4_class(res, "DataFrame")
+    expect_equal(colnames(res), "is_filled")
+    expect_true(nrow(res) == 0L)
+
+    expect_error(chromPeakData(xmse, columns = "a"), "valid column")
+})
